# baleen.console
# Implements the baleen console utility.
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Wed Mar 02 10:52:36 2016 -0500
#
# Copyright (C) 2016 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: __init__.py [] benjamin@bengfort.com $

"""
Implements the baleen console utility.
"""

##########################################################################
## Imports
##########################################################################

from .app import COMMANDS
from .app import BaleenUtility
