# baleen.utils
# Utilities and helpers functions for the Baleen project.
#
# Author:   Benjamin Bengfort <bengfort@cs.umd.edu>
# Created:  Sun Feb 21 15:00:06 2016 -0500
#
# Copyright (C) 2016 University of Maryland
# For license information, see LICENSE.txt
#
# ID: __init__.py [] benjamin@bengfort.com $

"""
Utilities and helpers functions for the Baleen project.
"""

##########################################################################
## Imports
##########################################################################

# For the log configuration to work
from . import mongolog
